"use strict";

{
    self.C3.Plugins.BHT_Smart_Random.Instance = class BHT_Smart_RandomInstance extends self.C3.SDKInstanceBase
    {
        constructor(inst, properties)
        {
            super(inst);
            console.log("BHT_Smart_RandomInstance:constructor-IN");

            this.StartValue = 0;
            this.EndValue = 0;
            this.Threshold = 0;
            this.DataWidth = 1;

            if (properties)
            {
                this.StartValue = properties[0];
                this.EndValue = properties[1];
                this.Threshold = properties[2];
            }

            this.DataWidth = (this.EndValue - this.StartValue) + 1;

            if (this.Threshold >= this.DataWidth)
            {
                // It isn't possible to have a threshold >= the data-width, so we will just ignore the value.
                console.log("Threshold is illegal. Ignoring it!");
                this.Threshold = 0;
            }

            this.DataArray = new Array(this.DataWidth);			// The actual randomized data
            this.TempArray = new Array(this.DataWidth);			// The initial sequential data that we randomly pull from.
            this.ThresholdArray = new Array(this.Threshold);	// A copy of the last data array so we can eliminate "threshold" duplicates.

            this.CurrentIndex = 0;

            this.Regenerate(true);
            console.log("BHT_Smart_RandomInstance:constructor-OUT");
        }

        Release()
        {
            super.Release();
        }

        Regenerate(firstTime)
        {
            console.log("Regenerate()");
            var index = 0;

            if (this.Threshold == 0)
            {
                firstTime = true;
            }
            // If this is our first time generating the data, or we don't have a threshold, just generate the full data range.
            if (!firstTime)
            {
                // We need to skip the last "threshold" number of entries.
                for (var i = 0; i < this.Threshold; i++)
                {
                    this.ThresholdArray[i] = this.DataArray[(this.DataWidth - this.Threshold) + i];
                }
            }

            // Start with every number in the range.
            for (var i = 0; i < this.DataWidth; i++)
            {
                this.TempArray[i] = i;
            }
            // Now randomly pull them out so that the final sequence is random.
            for (var i = 0; i < this.DataWidth; i++)
            {
                index = Math.floor(Math.random() * this.TempArray.length);
                this.DataArray[i] = this.TempArray[index];
                this.TempArray.splice(index, 1);
            }

            // Test against the previous data and swap duplicates beyond the threshold index.
            if (!firstTime)
            {
                var swapIndex = -1;

                for (var i = 0; i < this.ThresholdArray.length; i++)
                {
                    for (var j = 0; j < this.ThresholdArray.length; j++)
                    {
                        if (this.DataArray[j] == this.ThresholdArray[i])
                        {
                            // Now find a non-duplicate value beyond our threshold range.
                            var swapIndex = -1;
                            for (var k = this.ThresholdArray.length; k < this.DataArray.length && swapIndex == -1; k++)
                            {
                                var skip = 0;
                                for (var l = 0; l < this.ThresholdArray.length && swapIndex == -1; l++)
                                {
                                    if (this.DataArray[k] == this.ThresholdArray[l])
                                    {
                                        // This K value is in the Threshold array, skip it.
                                        l = this.ThresholdArray.length;
                                        skip = 1;
                                    }
                                }
                                if (skip == 0)
                                {
                                    swapIndex = k;
                                }
                            }
                            if (swapIndex != -1)
                            {
                                //
                                var tempData = this.DataArray[j];
                                this.DataArray[j] = this.DataArray[swapIndex];
                                this.DataArray[swapIndex] = tempData;
                            }
                        }
                    }
                }
            }

            this.CurrentIndex = 0;

            /*console.log("regen->");
            for(var i=0;i<this.DataWidth;i++)
            {
                console.log(i + ":" + this.DataArray[i]);
            }*/
        }

        SaveToJson()
        {
            var save = {
                "start": this.StartValue,
                "end": this.EndValue,
                "threshold": this.Threshold,
                "currentindex": this.CurrentIndex,
                "data": {}
            };
            //console.log("S:" + this.StartValue + "," + this.EndValue +"," + this.Threshold+","+this.CurrentIndex);

            for (var i = 0; i < this.DataArray.length; i++)
            {
                save["data"][i] = this.DataArray[i];
                //console.log("S:" + this.DataArray[i]);
            }

            return save;
        }

        LoadFromJson(o)
        {
            this.StartValue = o["start"];
            this.EndValue = o["end"];
            this.Threshold = o["threshold"];
            this.CurrentIndex = o["currentindex"];
            //console.log("L:" + this.StartValue + "," + this.EndValue + "," + this.Threshold + "," + this.CurrentIndex);

            this.DataWidth = (this.EndValue - this.StartValue) + 1;

            this.DataArray = [];
            for (var dt in o["data"])
            {
                this.DataArray[dt] = o["data"][dt];
                //console.log("L:"+this.DataArray[dt]);
            }
        }
    };
}